@@ -116,6 +116,7 @@ public:
   // Overloaded nsHTMLContainerFrame methods -- see documentation in nsIFrame.h
 
   virtual nsIAtom* GetType() const;
+  virtual PRBool IsFrameOfType(PRUint32 aFlags) const;
 
   NS_IMETHOD
   Init(nsPresContext*  aPresContext,
@@ -205,11 +206,6 @@ public:
   virtual nsresult
   ChildListChanged(PRInt32 aModType);
 
-  // helper to wrap non-MathML frames so that foreign elements (e.g., html:img)
-  // can mix better with other surrounding MathML markups
-  virtual nsresult
-  WrapForeignFrames();
-
   // helper to get the preferred size that a container frame should use to fire
   // the stretch on its stretchy child frames.
   virtual void
@@ -236,11 +232,14 @@ public:
               nsPresContext*          aPresContext,
               nsHTMLReflowMetrics&     aDesiredSize,
               const nsHTMLReflowState& aReflowState,
-              nsReflowStatus&          aStatus)
-  {
-    return nsHTMLContainerFrame::ReflowChild(aKidFrame, aPresContext, aDesiredSize, aReflowState,
-                                             0, 0, NS_FRAME_NO_MOVE_FRAME, aStatus);
-  }
+              nsReflowStatus&          aStatus);
+
+  nsresult 
+  ReflowForeignChild(nsIFrame*                aKidFrame,
+                     nsPresContext*           aPresContext,
+                     nsHTMLReflowMetrics&     aDesiredSize,
+                     const nsHTMLReflowState& aReflowState,
+                     nsReflowStatus&          aStatus);
 
   // helper to add the inter-spacing when <math> is the immediate parent.
   // Since we don't (yet) handle the root <math> element ourselves, we need to
