@@ -1,26 +1,43 @@
 # -*- Mode: C; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
-# The contents of this file are subject to the Netscape Public
-# License Version 1.1 (the "License"); you may not use this file
-# except in compliance with the License. You may obtain a copy of
-# the License at http://www.mozilla.org/NPL/
+# ***** BEGIN LICENSE BLOCK *****
+# Version: MPL 1.1/GPL 2.0/LGPL 2.1
 #
-# Software distributed under the License is distributed on an "AS
-# IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
-# implied. See the License for the specific language governing
-# rights and limitations under the License.
+# The contents of this file are subject to the Mozilla Public License Version
+# 1.1 (the "License"); you may not use this file except in compliance with
+# the License. You may obtain a copy of the License at
+# http://www.mozilla.org/MPL/
 #
-# The Original Code is Mozilla Communicator client code,
-# released March 31, 1998.
+# Software distributed under the License is distributed on an "AS IS" basis,
+# WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
+# for the specific language governing rights and limitations under the
+# License.
 #
-# The Initial Developer of the Original Code is Netscape Communications
-# Corporation.  Portions created by Netscape are
-# Copyright (C) 1998 Netscape Communications Corporation. All
-# Rights Reserved.
+# The Original Code is Mozilla Communicator client code, released
+# March 31, 1998.
+#
+# The Initial Developer of the Original Code is
+# Netscape Communications Corporation.
+# Portions created by the Initial Developer are Copyright (C) 1998
+# the Initial Developer. All Rights Reserved.
 #
 # Contributor(s):
-#     William A. ("PowerGUI") Law <law@netscape.com>
-#     Blake Ross <blakeross@telocity.com>
-#     Gervase Markham <gerv@gerv.net>
+#   William A. ("PowerGUI") Law <law@netscape.com>
+#   Blake Ross <blakeross@telocity.com>
+#   Gervase Markham <gerv@gerv.net>
+#
+# Alternatively, the contents of this file may be used under the terms of
+# either the GNU General Public License Version 2 or later (the "GPL"), or
+# the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
+# in which case the provisions of the GPL or the LGPL are applicable instead
+# of those above. If you wish to allow use of your version of this file only
+# under the terms of either the GPL or the LGPL, and not to allow others to
+# use your version of this file under the terms of the MPL, indicate your
+# decision by deleting the provisions above and replace them with the notice
+# and other provisions required by the GPL or the LGPL. If you do not delete
+# the provisions above, a recipient may use your version of this file under
+# the terms of any one of the MPL, the GPL or the LGPL.
+#
+# ***** END LICENSE BLOCK *****
 
 /*------------------------------ nsContextMenu ---------------------------------
 |   This JavaScript "class" is used to implement the browser's content-area    |
@@ -32,7 +49,6 @@
 |   longer term, this code will be restructured to make it more reusable.      |
 ------------------------------------------------------------------------------*/
 
-const mailtolength = 7;
 
 function nsContextMenu( xulMenu ) {
     this.target         = null;
@@ -40,6 +56,7 @@ function nsContextMenu( xulMenu ) {
     this.popupURL       = null;
     this.onTextInput    = false;
     this.onImage        = false;
+    this.onLoadedImage  = false;
     this.onLink         = false;
     this.onMailtoLink   = false;
     this.onSaveableLink = false;
@@ -116,7 +133,7 @@ nsContextMenu.prototype = {
         this.showItem( "context-savelink", this.onSaveableLink );
 
         // Save image depends on whether there is one.
-        this.showItem( "context-saveimage", this.onImage );
+        this.showItem( "context-saveimage", this.onLoadedImage );
         
         this.showItem( "context-sendimage", this.onImage );
     },
@@ -130,11 +147,11 @@ nsContextMenu.prototype = {
         this.showItem( "context-sep-properties", !( this.inDirList || this.isTextSelected || this.onTextInput ) );
         // Set As Wallpaper depends on whether an image was clicked on, and only works on Windows.
         var isWin = navigator.appVersion.indexOf("Windows") != -1;
-        this.showItem( "context-setWallpaper", isWin && this.onImage );
+        this.showItem( "context-setWallpaper", isWin && this.onLoadedImage );
 
         this.showItem( "context-sep-image", this.onImage );
 
-        if( isWin && this.onImage )
+        if( isWin && this.onLoadedImage )
             // Disable the Set As Wallpaper menu item if we're still trying to load the image
           this.setItemAttr( "context-setWallpaper", "disabled", (("complete" in this.target) && !this.target.complete) ? "true" : null );
 
@@ -225,6 +242,7 @@ nsContextMenu.prototype = {
         }
         // Initialize contextual info.
         this.onImage    = false;
+        this.onLoadedImage = false;
         this.onStandaloneImage = false;
         this.onMetaDataItem = false;
         this.onTextInput = false;
@@ -240,69 +258,18 @@ nsContextMenu.prototype = {
 
         // See if the user clicked on an image.
         if ( this.target.nodeType == Node.ELEMENT_NODE ) {
-             if ( this.target instanceof HTMLImageElement ) {
+            if ( this.target instanceof Components.interfaces.nsIImageLoadingContent && this.target.currentURI  ) {
                 this.onImage = true;
-                this.imageURL = this.target.src;
+                var request = this.target.getRequest( Components.interfaces.nsIImageLoadingContent.CURRENT_REQUEST );
+                if (request && (request.imageStatus & request.STATUS_SIZE_AVAILABLE))
+                    this.onLoadedImage = true;
+                this.imageURL = this.target.currentURI.spec;
 
-                var documentType = window._content.document.contentType;
-                if ( documentType.substr(0,6) == "image/" )
-                    this.onStandaloneImage = true;
-
-                // Look for image map.
-                var mapName = this.target.getAttribute( "usemap" );
-                if ( mapName ) {
-                    // Find map.
-                    var map = this.target.ownerDocument.getElementById( mapName.substr(1) );
-                    if ( map ) {
-                        // Search child <area>s for a match.
-                        var areas = map.childNodes;
-                        //XXX Client side image maps are too hard for now!
-                        areas.length = 0;
-                        for ( var i = 0; i < areas.length && !this.onLink; i++ ) {
-                            var area = areas[i];
-                            if ( area instanceof HTMLAreaElement ) {
-                                // Get type (rect/circle/polygon/default).
-                                var type = area.getAttribute( "type" );
-                                var coords = this.parseCoords( area );
-                                switch ( type.toUpperCase() ) {
-                                    case "RECT":
-                                    case "RECTANGLE":
-                                        break;
-                                    case "CIRC":
-                                    case "CIRCLE":
-                                        break;
-                                    case "POLY":
-                                    case "POLYGON":
-                                        break;
-                                    case "DEFAULT":
-                                        // Default matches entire image.
-                                        this.onLink = true;
-                                        this.link = area;
-                                        this.onSaveableLink = this.isLinkSaveable( this.link );
-                                        break;
-                                }
-                            }
-                        }
-                    }
-                }
-             } else if ( this.target instanceof HTMLObjectElement
-                         &&
-                         // See if object tag is for an image.
-                         this.objectIsImage( this.target ) ) {
-                // This is an image.
-                this.onImage = true;
-                // URL must be constructed.
-                this.imageURL = this.objectImageURL( this.target );
-             } else if ( this.target instanceof HTMLInputElement ) {
+                if ( this.target.ownerDocument instanceof ImageDocument )
+                   this.onStandaloneImage = true;
+            } else if ( this.target instanceof HTMLInputElement) {
                type = this.target.getAttribute("type");
-               if(type && type.toUpperCase() == "IMAGE") {
-                 this.onImage = true;
-                 // Convert src attribute to absolute URL.
-                 this.imageURL = this.makeURLAbsolute( this.target.baseURI,
-                                                       this.target.src );
-               } else /* if (this.target.getAttribute( "type" ).toUpperCase() == "TEXT") */ {
-                 this.onTextInput = this.isTargetATextBox(this.target);
-               }
+               this.onTextInput = this.isTargetATextBox(this.target);
             } else if ( this.target instanceof HTMLTextAreaElement ) {
                  this.onTextInput = true;
             } else if ( this.target instanceof HTMLHtmlElement ) {
@@ -404,7 +371,6 @@ nsContextMenu.prototype = {
                              ( elem.cite || elem.dateTime ) )                ||
                          ( elem instanceof HTMLElement &&
                              ( elem.title || elem.lang ) ) ) {
-                        dump("On metadata item.\n");
                         this.onMetaDataItem = true;
                     }
                 }
@@ -604,14 +570,30 @@ nsContextMenu.prototype = {
         // Copy the comma-separated list of email addresses only.
         // There are other ways of embedding email addresses in a mailto:
         // link, but such complex parsing is beyond us.
+        
+        const kMailToLength = 7; // length of "mailto:"
+
         var url = this.linkURL();
         var qmark = url.indexOf( "?" );
         var addresses;
         
-        if ( qmark > mailtolength ) 
-            addresses = url.substring( mailtolength, qmark );
-        else 
-            addresses = url.substr( mailtolength );
+        if ( qmark > kMailToLength ) {
+            addresses = url.substring( kMailToLength, qmark );
+        } else {
+            addresses = url.substr( kMailToLength );
+        }
+
+        // Let's try to unescape it using a character set
+        try {
+          var ownerDocument = new XPCNativeWrapper(this.target, "ownerDocument").ownerDocument;
+          var characterSet = new XPCNativeWrapper(ownerDocument, "characterSet").characterSet;
+          const textToSubURI = Components.classes["@mozilla.org/intl/texttosuburi;1"]
+                                         .getService(Components.interfaces.nsITextToSubURI);
+          addresses = textToSubURI.unEscapeURIForUI(characterSet, addresses);
+        }
+        catch(ex) {
+          // Do nothing.
+        }
 
         var clipboard = this.getService( "@mozilla.org/widget/clipboardhelper;1",
                                          Components.interfaces.nsIClipboardHelper );
@@ -621,7 +603,7 @@ nsContextMenu.prototype = {
       var docshell = document.getElementById( "content" ).webNavigation;
       BookmarksUtils.addBookmark( docshell.currentURI.spec,
                                   docshell.document.title,
-                                  docshell.document.charset,
+                                  docshell.document.characterSet,
                                   false );
     },
     addBookmarkForFrame : function() {
@@ -632,7 +614,7 @@ nsContextMenu.prototype = {
         title = uri;
       BookmarksUtils.addBookmark( uri,
                                   title,
-                                  doc.charset,
+                                  doc.characterSet,
                                   false );
     },
     // Open Metadata window for node
@@ -763,7 +745,7 @@ nsContextMenu.prototype = {
     
     searchSelected : function() {
         var focusedWindow = document.commandDispatcher.focusedWindow;
-        var searchStr = focusedWindow.getSelection();
+        var searchStr = focusedWindow.getSelection();;
         searchStr = searchStr.toString();
         searchStr = searchStr.replace( /^\s+/, "" );
         searchStr = searchStr.replace(/(\n|\r|\t)+/g, " ");
@@ -771,26 +753,6 @@ nsContextMenu.prototype = {
         return searchStr;
     },
     
-    // Determine if target <object> is an image.
-    objectIsImage : function ( objElem ) {
-        var result = false;
-        // Get type and data attributes.
-        var type = objElem.getAttribute( "type" );
-        var data = objElem.getAttribute( "data" );
-        // Presume any mime type of the form "image/..." is an image.
-        // There must be a data= attribute with an URL, also.
-        if ( type.substring( 0, 6 ) == "image/" && data && data != "" ) {
-            result = true;
-        }
-        return result;
-    },
-    // Extract image URL from <object> tag.
-    objectImageURL : function ( objElem ) {
-        // Extract url from data= attribute.
-        var data = objElem.getAttribute( "data" );
-        // Make it absolute.
-        return this.makeURLAbsolute( objElem.baseURI, data );
-    },
     // Convert relative URL to absolute, using document's <base>.
     makeURLAbsolute : function ( base, url ) {
         // Construct nsIURL.
@@ -801,10 +763,6 @@ nsContextMenu.prototype = {
         return ioService.newURI(baseURI.resolve(url), null, null).spec;
     },
 
-    // Parse coords= attribute and return array.
-    parseCoords : function ( area ) {
-        return [];
-    },
     toString : function () {
         return "contextMenu.target     = " + this.target + "\n" +
                "contextMenu.onImage    = " + this.onImage + "\n" +
@@ -851,7 +809,7 @@ function nsDefaultEngine()
         var pb = Components.classes["@mozilla.org/preferences-service;1"].
                    getService(Components.interfaces.nsIPrefBranch);
         var pbi = pb.QueryInterface(
-                    Components.interfaces.nsIPrefBranchInternal);
+                    Components.interfaces.nsIPrefBranch2);
         pbi.addObserver(this.domain, this, false);
 
         // reuse code by explicitly invoking initial |observe| call
